/**
 * @brief  PD_HVInfo.h
 * @param
 * @author lzc
 * @date 2021-11-22
 * @version 1.0
 * @property  to update behavior node HVInfo and node collision status
 * */
#pragma once

#include "PD_RoadModel_CarPosition_PDtoBA.h"
#include "PD_EgoVehStsArb.h"
#include "PD_SVEstmd.h"
#include "PD_Input.h"
#include <algorithm>
#include <math.h>
#include "../../source/PD_commonfunc.h"
#include <algorithm>
#include "../../../../include/CtApBehaviorAnalysis_/BA_Enum_List.h"

namespace ADC
{
    namespace PD_NodeUpdate
    {
        struct PD_ClsnSVObj
        {
            uint32_t m_ObjID = 0;
            std::vector<float> vec_l;
            std::vector<float> vec_relat_s;
            std::vector<float> vec_vs;
        };

        struct PD_NodeHVInfo
        {
            float t;            // time
            float relat_s;      // lgtdist
            float re2ref_l;     // lateral distance
            float vx;           // lgtspd
            uint8_t locationID; // vehicle of Lane ID
            float RELAT_S;      // refer to each node relative position
        };

        struct refC0Info
        {
            float endrange = 0.0f;
            float refC0 = 0.0f;
            float relat_s = 0.0f;
        };

        struct DstntionLaneMkID
        {
            uint32_t LftMk_id = 0;
            uint32_t RgtMk_id = 0;
        };

        struct ClsnObjInfo
        {
            bool ClsnFlag = false;
            uint32_t ClsnObjID = 0;
        };

        float GetObjSizeWidth(const uint32_t &objid,
                              const PD::PD_Input &pd_input); // according to obj id to get width

        bool Init(const PD_EgoVehTrj::EgoVehTrj &egovehtrj,
                  const PD_SVEstmd::SVEstmd &svtrajpnt);
        

        class NodeUpdate
        {
        public:
            NodeUpdate() = default;

            void ResDurationHVInfo(const PD_EgoVehTrj::EgoVehTrj &egovehtrj,
                                   const PD::PD_Input &pd_input);
            bool NodeClsnStsCalu(const PD_SVEstmd::SVEstmd &svtrajpnt,
                                 const PD::PD_Input &pd_input,
                                 const PD_EgoVehTrj::EgoVehTrj &egovehtrj);

            // bool Init(const PD::PD_Input &pd_input);

            std::vector<PD_NodeHVInfo> nodehvinfo;

            PD_NodeHVInfo pd_nodehvinfo;

            std::vector<PD_ClsnSVObj> PD_NodeObjStsVec;
            std::vector<refC0Info> _related_refC0; // to store related boundary line geometry c0;

            void GetNodeRelatS(const PD_EgoVehTrj::EgoVehTrj &egovehtrj);

            bool NodeClsnFlag = false;

            void StnEnvLeadClsn(const PD::PD_Input &pd_input);
            void LessLaneWidLeadClsn(const PD::PD_Input &pd_input);
            void RoadEndLeadClsn(const PD::PD_Input &pd_input);
            void RoadEndLeadClsnProc(const PD::PD_Input &pd_input);
            uint8_t CorIDMatchLaneID(const PD::PD_Input &pd_input); // match corridor id to lane id
            float CrsLineTrjPntProc();                              // to get lgt cross line trajectory point

            bool RoadEndProc(const PD::PD_Input &pd_input); // to classify city scenario
            bool RoadEnd_outofRge = true;

            bool ClsnFlg2Attention = false; // to analysis dynamic object risk
            std::vector<ClsnObjInfo> clsnobjinfo;

            // add funciton to process temporal sequence case
            void NodeObjChkClsnProc();
            float GetNodeMinDist(std::vector<float> & pd_clsnobj);

            bool VehLaneChgStsProc(const PD::PD_Input &pd_input);  // judege vehicle lane change or not at current time
        private:
            
            bool LaneChgClsnFlag = false;

            bool CrsSldLineClsnFlag = false;
            bool CrsLaneClsnFlag = false;
            bool InsfcntLaneWidClsnFlag = false;
            bool RdEndClsnFlag = false;

            std::vector<uint32_t> NeightborLaneObjID;

            //    const PD::PD_Input &pd_input);
        };
    } // namespace PD_NodeUpdate

} // namespace ADC
